﻿#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ftd2xx.h"
#include "gto2param.h"

void usage(void){
  printf("hints\n");
  printf("a. run as root\n");
  printf("b. if ftdi_sio driver is loaded, please remove\n");
  printf("  1. /sbin/lsmod | grep ftdi_sio (if there is)\n");
  printf("  2. /sbin/rmmod ftdi_sio\n");
  printf("\n");
}


int main(int argc, char *argv[]){
	FT_HANDLE ftHandle = NULL;
	FT_STATUS ftStatus;
	FILE* fd = NULL;

	char *dbuff;
	char obuff[512], ibuff[256];
	int size, ret, i, idx=0;
	int wcnt = 0, rcnt = 0;
	int clkdiv = 10;
	char cs_bit = 0x18;
	char cs_pin = 0x1b;
	char rst_bit = 0x08;
	char rst_pin = 0x08;

	int numDevs;
	FT_DEVICE_LIST_INFO_NODE *devInfo;

	dbuff = malloc(512 * 1024);
	memset(dbuff, 0, 512 * 1024);
	fd = NULL;

	FT_SetVIDPID(VID, PID);

	ftStatus = FT_CreateDeviceInfoList(&numDevs);
	if(ftStatus != FT_OK){
	  printf("error in create device info list\n");
	  usage();
	  return 0;
	}
	if(numDevs <= 0){
	  printf("Cannot find GTO2 USB Interface\n");
	  usage();
	  return 0;
	}else{
	  devInfo = (FT_DEVICE_LIST_INFO_NODE*)malloc(sizeof(FT_DEVICE_LIST_INFO_NODE)*numDevs);
	  ftStatus = FT_GetDeviceInfoList(devInfo, &numDevs);
	  for(i=0;i<numDevs;i++){
	    printf("Dev %d:\n", i);
	    printf(" Flags=0x%x\n",devInfo[i].Flags);
	    printf(" Type=0x%x\n",devInfo[i].Type);
	    printf(" ID=0x%x\n",devInfo[i].ID);
	    printf(" LocId=0x%x\n",devInfo[i].LocId);
	    printf(" SerialNumber=%s\n",devInfo[i].SerialNumber);
	    printf(" Description=%s\n",devInfo[i].Description);
	    printf(" ftHandle=0x%x\n",devInfo[i].ftHandle); 
	  }
	}


	ftStatus = FT_Open(1, &ftHandle);
	if (ftStatus != FT_OK) {
	  printf("Cannot find GTO2 USB\n");
	  exit(1);
	}

	printf("ft opend\n");

	
	ftStatus |= FT_ResetDevice(ftHandle);
	ftStatus |= FT_GetQueueStatus(ftHandle, &size);
	if ((ftStatus == FT_OK) && (size > 0)) {
		FT_Read(ftHandle, &ibuff, size, &ret);
	}
	ftStatus |= FT_SetUSBParameters(ftHandle, 512, 512);
	ftStatus |= FT_SetChars(ftHandle, 0, 0, 0, 0);
	ftStatus |= FT_SetTimeouts(ftHandle, 0, 5000);
	ftStatus |= FT_SetLatencyTimer(ftHandle, 2);
	ftStatus |= FT_SetFlowControl(ftHandle, FT_FLOW_NONE, 0x00, 0x00);
	ftStatus |= FT_SetBitMode(ftHandle, 0x0, 0x00);
	ftStatus |= FT_SetBitMode(ftHandle, 0x0, 0x02);
	if (ftStatus != FT_OK) {
		printf("Error in initializing the MPSSE %d\n", ftStatus);
		FT_Close(ftHandle);
		return 1; // Exit with error
	}
	usleep(50000); // Wait for all the USB stuff to complete and work


	size = 0;
	obuff[size++] = 0x8A;
	// Use 60MHz master clock (disable divide by 5)
	obuff[size++] = 0x97;
	// Turn off adaptive clocking (may be needed for ARM)
	obuff[size++] = 0x8D;
	// Disable three-phase clocking
	ftStatus = FT_Write(ftHandle, obuff, size, &ret);

	size = 0;
	obuff[size++] = 0x86;
	obuff[size++] = clkdiv & 0xff;
	obuff[size++] = (clkdiv >> 8) & 0xff;
	ftStatus = FT_Write(ftHandle, obuff, size, &ret);

	size = 0;
	obuff[size++] = 0x85;
	// loop back end
	ftStatus = FT_Write(ftHandle, obuff, size, &ret);

	// pin setting low (spi)
	size = 0;
	obuff[size++] = 0x80;
	obuff[size++] = cs_bit;
	obuff[size++] = cs_pin;
	ftStatus = FT_Write(ftHandle, obuff, size, &ret);

	// pin setting high (prog_b=1);
	size = 0;
	//obuff[size++] = 0x82;
	//obuff[size++] = rst_bit;
	//obuff[size++] = rst_pin;
	obuff[size++] = 0x82;
	obuff[size++] = 0 & ~rst_bit;
	obuff[size++] = rst_pin;
	ftStatus = FT_Write(ftHandle, obuff, size, &ret);


	// do write and read
	wcnt = 0;
	rcnt = 0;
	

	// get ID
	size = 0;
	obuff[size++] = 0x80;
	obuff[size++] = 0 & ~cs_bit;
	obuff[size++] = cs_pin;

	wcnt = 1;
	obuff[size++] = 0x11; // Do write with Neg edge
	obuff[size++] = (wcnt - 1) & 0xff;
	obuff[size++] = ((wcnt - 1) >> 8) & 0xff;
	obuff[size++] = 0x9f;   // Read Manufacturer ID
//	obuff[size++] = 0xd7;   // Read Status Register

	rcnt = 5;
//	rcnt = 2;
	if (rcnt) {
		obuff[size++] = 0x24; // Do read with Neg eddge
		obuff[size++] = (rcnt - 1) & 0xff;
		obuff[size++] = ((rcnt - 1) >> 8) & 0xff;
		ftStatus = FT_Write(ftHandle, obuff, size, &ret);
		size = 0;

		ftStatus = FT_Read(ftHandle, ibuff, rcnt, &ret);
		printf("FT Status=%d\n", ftStatus);
		printf("Read count = %d\n", ret);
		for (i = 0; i < rcnt; i++) {
			printf("%d : %02x\n", i, (unsigned char)ibuff[i]);
		}
	}


	// pin setting low (spi)
	size = 0;
	obuff[size++] = 0x80;
	obuff[size++] = cs_bit;
	obuff[size++] = cs_pin;
	obuff[size++] = 0x82;
	obuff[size++] = rst_bit;
	obuff[size++] = rst_pin;
	ftStatus = FT_Write(ftHandle, obuff, size, &ret);
	usleep(50000);  // wait for program and erase (50ms)



	if (ftHandle != NULL) {
		FT_Close(ftHandle);
	}


	return 0;
}

