﻿#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "ftd2xx.h"
#include "gto2param.h"

int main(int argc, char *argv[]){
	FT_HANDLE ftHandle = NULL;
	FT_STATUS ftStatus;

	int numDevs;
	int i, step=0;
	char yn = 0;

	FT_PROGRAM_DATA ftData;
	char ManufacturerBuf[32];
	char ManufacturerIdBuf[16];
	char DescriptionBuf[64];
	char SerialNumberBuf[16];

	ftData.Signature1 = 0x00000000;
	ftData.Signature2 = 0xffffffff;
	ftData.Version = 0x00000005;
	ftData.Manufacturer = ManufacturerBuf;
	ftData.ManufacturerId = ManufacturerIdBuf;
	ftData.Description = DescriptionBuf;
	ftData.SerialNumber = SerialNumberBuf;


	printf("*** GTO2 USB Inteface setup program ***\n");
	printf(" please make sure that\n");
	printf(" a. run as root\n");
	printf(" b. if ftdi_sio driver is loaded, please remove\n");
	printf("  1. /sbin/lsmod | grep ftdi_sio (if there is)\n");
	printf("  2. /sbin/rmmod ftdi_sio\n");
	printf(" c. if try to re-setup GTO2 USB Interface, please use -u option (gto2eep -u)\n");
	printf("\n PLEASE ENTER \n");
	getchar();

	if(argc >1){
	  if(!strcmp(argv[1], "-u")){
	    printf(" -- re-setup mode --\n");
	    FT_SetVIDPID(VID, PID);
	  }
	}


	ftStatus = FT_CreateDeviceInfoList((LPDWORD)&numDevs);
	if(ftStatus != FT_OK){
	  printf("error in create device info list\n");
	  return 0;
	}
	if(numDevs <= 0){
	  printf("Cannot find GTO2 USB Interface\n");
	  return 0;
	}else{
	  /* noop */
	}

	step = 0;
	yn = 0;
	for(i=0;i<numDevs;i++){
	  ftHandle = NULL;
	  ftStatus = FT_Open(i, &ftHandle);
	  if (ftStatus != FT_OK) {
	    continue;
	  }

	  printf("ft opend device:%d\n", i);

	  if(step == 0){
	    ftStatus = FT_EE_Read(ftHandle, &ftData);
	    if(ftStatus != FT_OK){
	      if(ftStatus == FT_EEPROM_NOT_PROGRAMMED){
		printf("Blank EEPROM is detected\n");
		printf("Initialize GTO2 EEPROM for this device ? [y/n] : ");
		yn = getchar();
		printf("\n");
	      }else{
		printf(" no EEPROM, skip this device\n\n");
		continue;
	      }
	    }else{
	      printf("Signature1     : 0x%08x\n", ftData.Signature1);
	      printf("Signature2     : 0x%08x\n", ftData.Signature2);
	      printf("Manufacturer   : %s\n", ManufacturerBuf);
	      printf("ManufacturerId : %s\n", ManufacturerIdBuf);
	      printf("Description    : %s\n", DescriptionBuf);
	      printf("SerialNumber   : %s\n", SerialNumberBuf);
	      printf("Version        : %d\n", ftData.Version);
	      
	      if(ftData.Version != 3){
		printf("This device is not for GTO2 (skip)\n\n");
		continue;
	      }
	      printf("Initialize GTO2 EEPROM for this device ? [y/n] : ");
	      yn = getchar();
	      printf("\n");
	    }
	    if(yn == 'y'){
	      ftStatus = FT_EE_Program(ftHandle, &gtospiData);
	      if(ftStatus == FT_OK){
		printf(" GTO2SPI Interface Program OK\n");
		step = 1;
	      }else{
		printf(" Failed to program\n");
		return 0;
	      }
	    }else{
	      continue;
	    }
	  }else if(step == 1){
	    ftStatus = FT_EE_Program(ftHandle, &gtousbData);
	    if(ftStatus == FT_OK){
	      printf(" GTO2DATA Interface Program OK\n");
	      printf("\n **** Please power cycle GTO2 **** \n");
	    }else{
	      printf(" Failed to program\n");
	      return 0;
	    }
	  }
	  
	  
	  if (ftHandle != NULL) {
	    FT_Close(ftHandle);
	    usleep(100000);
	  }
	}




	return 0;
}

