﻿#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "ftd2xx.h"
#include "gto2param.h"

void usage(void){
  printf("**** hints ****\n");
  printf("a. run as root\n");
  printf("b. setup GTO2USB Inteface by gto2eep\n");
  printf("\n");
}

int main(int argc, char *argv[]){
	FT_HANDLE ftHandle = NULL;
	FT_STATUS ftStatus;
	FILE* fd = NULL;

	char *dbuff;
	char obuff[512], ibuff[256];
	int size, ret, i;
	int wcnt = 0;
	int clkdiv = 10;
	char cs_bit = 0x18;
	char cs_pin = 0x1b;
	char rst_bit = 0x08;
	char rst_pin = 0x08;

	if (argc != 2) {
	  printf("gto2writer FILENAME\n");
	  //		exit(0);
	}

	fd = fopen(argv[1], "rb");
	if (fd == NULL) {
	  printf("Cannot open bit file\n");
	  exit(1);
	}

	dbuff = malloc(512 * 1024);
	memset(dbuff, 0, 512 * 1024);
	fread(dbuff, 1, 512 * 1024, fd);
	fclose(fd);
	fd = NULL;


	FT_SetVIDPID(VID, PID);

	ftStatus = FT_OpenEx(GTO2USBDESC, FT_OPEN_BY_DESCRIPTION, &ftHandle);
	if (ftStatus != FT_OK) {
	  printf("Cannot find GTO2 SPI\n");
	  usage();
	  exit(1);
	}


	printf("GTO2 opend\n");

	
	ftStatus |= FT_ResetDevice(ftHandle);
	ftStatus |= FT_GetQueueStatus(ftHandle, (LPDWORD)&size);
	if ((ftStatus == FT_OK) && (size > 0)) {
	  FT_Read(ftHandle, &ibuff, (DWORD)size, (LPDWORD)&ret);
	}
	ftStatus |= FT_SetUSBParameters(ftHandle, 512, 512);
	ftStatus |= FT_SetChars(ftHandle, 0, 0, 0, 0);
	ftStatus |= FT_SetTimeouts(ftHandle, 0, 5000);
	ftStatus |= FT_SetLatencyTimer(ftHandle, 2);
	ftStatus |= FT_SetFlowControl(ftHandle, FT_FLOW_NONE, 0x00, 0x00);
	ftStatus |= FT_SetBitMode(ftHandle, 0x0, 0x00);
	ftStatus |= FT_SetBitMode(ftHandle, 0x0, 0x02);
	if (ftStatus != FT_OK) {
	  printf("Error in initializing the MPSSE %d\n", ftStatus);
	  FT_Close(ftHandle);
	  return 1; // Exit with error
	}
	usleep(50000); // Wait for all the USB stuff to complete and work


	size = 0;
	obuff[size++] = 0x8A;
	// Use 60MHz master clock (disable divide by 5)
	obuff[size++] = 0x97;
	// Turn off adaptive clocking (may be needed for ARM)
	obuff[size++] = 0x8D;
	// Disable three-phase clocking
	ftStatus = FT_Write(ftHandle, obuff, (DWORD)size, (LPDWORD)&ret);

	size = 0;
	obuff[size++] = 0x86;
	obuff[size++] = clkdiv & 0xff;
	obuff[size++] = (clkdiv >> 8) & 0xff;
	ftStatus = FT_Write(ftHandle, obuff, (DWORD)size, (LPDWORD)&ret);

	size = 0;
	obuff[size++] = 0x85;
	// loop back end
	ftStatus = FT_Write(ftHandle, obuff, (DWORD)size, (LPDWORD)&ret);

	// pin setting low (spi)
	size = 0;
	obuff[size++] = 0x80;
	obuff[size++] = cs_bit;
	obuff[size++] = cs_pin;
	ftStatus = FT_Write(ftHandle, obuff, (DWORD)size, (LPDWORD)&ret);

	// pin setting high (prog_b=1);
	size = 0;
	//obuff[size++] = 0x82;
	//obuff[size++] = rst_bit;
	//obuff[size++] = rst_pin;
	obuff[size++] = 0x82;
	obuff[size++] = 0 & ~rst_bit;
	obuff[size++] = rst_pin;
	ftStatus = FT_Write(ftHandle, obuff, (DWORD)size, (LPDWORD)&ret);


	// do write and read
	wcnt = 0;
	

	// Chip write
	for (i = 0; i < 2048; i++) {
	  if (i % 100 == 0) {
	    printf("now writing page (%d/2048)\n", i+1);
	  }
	  size = 0;
	  obuff[size++] = 0x80;
	  obuff[size++] = 0 & ~cs_bit;
	  obuff[size++] = cs_pin;
	  
	  wcnt = 256 + 4; // 256 data + 1 code + 3 addr
	  obuff[size++] = 0x11; // Do write with Neg edge
	  obuff[size++] = (wcnt - 1) & 0xff;
	  obuff[size++] = ((wcnt - 1) >> 8) & 0xff;
	  //
	  obuff[size++] = 0x82; // program w/ erase
	  obuff[size++] = ((i & 0x0700) >> 8) & 0xff;
	  obuff[size++] = i & 0xff;
	  obuff[size++] = 0;
	  memcpy(obuff + size, dbuff + i * 256, 256);
	  size += 256;
	  //
	  obuff[size++] = 0x80;
	  obuff[size++] = cs_bit;
	  obuff[size++] = cs_pin;
	  ftStatus = FT_Write(ftHandle, obuff, (DWORD)size, (LPDWORD)&ret);
	  usleep(50000);  // wait for program and erase (50ms)
	}
	
	// read all
	/*
	fopen_s(&fd, "rev.bit", "wb");
	for (i = 0; i < 2048; i++) {
	size = 0;
	obuff[size++] = 0x80;
	obuff[size++] = 0 & ~cs_bit;
	obuff[size++] = cs_pin;
	wcnt = 8;
	obuff[size++] = 0x11; // Do write with Neg edge
	obuff[size++] = (wcnt - 1) & 0xff;
	obuff[size++] = ((wcnt - 1) >> 8) & 0xff;
	obuff[size++] = 0xd2;   // page read
	obuff[size++] = ((i & 0x0700) >> 8) & 0xff;
	obuff[size++] = i & 0xff;
	obuff[size++] = 0;
	obuff[size++] = 0; // dummy
	obuff[size++] = 0; // dummy
	obuff[size++] = 0; // dummy
	obuff[size++] = 0; // dummy
	
	rcnt = 256;
	obuff[size++] = 0x24; // Do read with Neg eddge
	obuff[size++] = (rcnt - 1) & 0xff;
	obuff[size++] = ((rcnt - 1) >> 8) & 0xff;
	
	ftStatus = FT_Write(ftHandle, obuff, size, &ret);
	size = 0;
	
	ftStatus = FT_Read(ftHandle, ibuff, rcnt, &ret);
	fwrite(ibuff, 1, 256, fd);
	
	size = 0;
	obuff[size++] = 0x80;
	obuff[size++] = cs_bit;
	obuff[size++] = cs_pin;
	ftStatus = FT_Write(ftHandle, obuff, size, &ret);
	Sleep(10);
	printf("%d\n", i);
	}
	
	fclose(fd);
*/
 
	
/*
	// get ID
	size = 0;
	obuff[size++] = 0x80;
	obuff[size++] = 0 & ~cs_bit;
	obuff[size++] = cs_pin;

	wcnt = 1;
	obuff[size++] = 0x11; // Do write with Neg edge
	obuff[size++] = (wcnt - 1) & 0xff;
	obuff[size++] = ((wcnt - 1) >> 8) & 0xff;
//	obuff[size++] = 0x9f;   // Read Manufacturer ID
	obuff[size++] = 0xd7;   // Read Status Register

//	rcnt = 5;
	rcnt = 2;
	if (rcnt) {
	obuff[size++] = 0x24; // Do read with Neg eddge
	obuff[size++] = (rcnt - 1) & 0xff;
	obuff[size++] = ((rcnt - 1) >> 8) & 0xff;
	ftStatus = FT_Write(ftHandle, obuff, size, &ret);
	size = 0;
	
	ftStatus = FT_Read(ftHandle, ibuff, rcnt, &ret);
	printf("FT Status=%d\n", ftStatus);
	printf("Read count = %d\n", ret);
	for (i = 0; i < rcnt; i++) {
	printf("%d : %02x\n", i, (unsigned char)ibuff[i]);
	}
	}
*/	

	// pin setting low (spi)
	size = 0;
	obuff[size++] = 0x80;
	obuff[size++] = cs_bit;
	obuff[size++] = cs_pin;
	obuff[size++] = 0x82;
	obuff[size++] = rst_bit;
	obuff[size++] = rst_pin;
	ftStatus = FT_Write(ftHandle, obuff, (DWORD)size, (LPDWORD)&ret);

	usleep(50000);  // wait for program and erase (50ms)



	if (ftHandle != NULL) {
	  FT_Close(ftHandle);
	}


	return 0;
}

